/***************************************************************************
 *   Copyright (C) 2006 by Ken Werner                                      *
 *   ken.werner@web.de                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "source.h"
#include <math.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>

Source::Source(QWidget* inParent):	
		mID(""),
		mName(""),
		mDescription(""),
		mEnabled(true),
		mShowName(true),
		mToolTipEnabled(true),
		mSourcePrefs(0){
}

Source::~Source(){
}

const QString& Source::getID() const{
	return mID;
}

const QString& Source::getName() const{
	return mName;
}

const QString& Source::getDescription() const{
	return mDescription;
}

bool Source::isEnabled() const{
	return mEnabled;
}

bool Source::isToolTipEnabled() const{
	return mToolTipEnabled;
}

bool Source::showName() const{
	return mShowName;
}

void Source::setEnabled(bool inEnabled){
	if(mEnabled != inEnabled){
		mEnabled = inEnabled;
		emit enabledChanged(mEnabled, getWidget());
	}
}

QWidget* Source::createPrefs(QWidget* inParent){
	if(!mSourcePrefs){
		mSourcePrefs = new SourcePrefs(inParent, "sourceprefsui");
		
		// disable nameCheckBox if enable source checkbox is disabled
		connect(mSourcePrefs->enabledCheckBox, SIGNAL(toggled(bool)), mSourcePrefs->nameCheckBox, SLOT(setEnabled(bool)));
		
		// add prefs widgets from sub classes
		QWidget* subPrefs = addPrefs(mSourcePrefs);
		if(subPrefs != NULL)
			mSourcePrefs->layout()->add(subPrefs);
			
		// add bottom vspacer
    	mSourcePrefs->layout()->addItem(new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding) );

		updatePrefsGUI(); // fill the prefs gui
	}
	return mSourcePrefs;
}

SourcePrefs* Source::getPrefs(){
	return mSourcePrefs;
}

void Source::updatePrefsGUI(){
	// set values
	mSourcePrefs->nameLineEdit->setText(mName);
	mSourcePrefs->descriptionLabel->setText(mDescription);
	mSourcePrefs->enabledCheckBox->setChecked(mEnabled);
	mSourcePrefs->nameCheckBox->setChecked(mShowName);
	mSourcePrefs->tooltipCheckBox->setChecked(mToolTipEnabled);
	// disable if source is disabled
	mSourcePrefs->nameCheckBox->setEnabled(mEnabled);
}

QWidget* Source::addPrefs(QWidget* inParent){
	return NULL;
}

void Source::applyPrefs(){
	setEnabled(mSourcePrefs->enabledCheckBox->isChecked());
	mShowName = mSourcePrefs->nameCheckBox->isChecked();
	mName = mSourcePrefs->nameLineEdit->text();
	mToolTipEnabled = mSourcePrefs->tooltipCheckBox->isChecked();
}

void Source::savePrefs(KConfig* inKConfig){
	applyPrefs();
	inKConfig->writeEntry(mID + "_enabled", mEnabled);
	inKConfig->writeEntry(mID + "_showName", mShowName);
	inKConfig->writeEntry(mID + "_name", mName);
	inKConfig->writeEntry(mID + "_toolTipEnabled", mToolTipEnabled);
}

void Source::loadPrefs(KConfig* inKConfig){
	setEnabled(inKConfig->readBoolEntry(mID + "_enabled", mEnabled));
	mShowName = inKConfig->readBoolEntry(mID + "_showName", mShowName);
	mName = inKConfig->readEntry(mID + "_name", mName);
	mToolTipEnabled = inKConfig->readBoolEntry(mID + "_toolTipEnabled", mToolTipEnabled);
}
